#ifndef _TCPKILL_H
#define _TCPKILL_H


/* Checksum stuff */
#define LIBNET_CKSUM_CARRY(x) \
    (x = (x >> 16) + (x & 0xffff), (~(x + (x >> 16)) & 0xffff))


/* Ethernet header */
#define ETH_HDRSIZE      sizeof(struct ether_header)
#define	ETH_TYPE_ARP     0x0806  /* Ether type ARP               */
#define	ETH_TYPE_IP      0x0800  /* Ether type IP                */
#define ETH_ADD_LEN      6       /* Ethernet address length      */
#define ETH_PADDING_ARP  18      /* 18 bytes ethernet padding    */

/* IP header */
#define IP_HDRSIZE       sizeof(struct ip_header)
#define IP_PRO_ICMP      0x01    /* IP protocol ICMP             */
#define IP_PRO_TCP       0x06    /* IP protocol TCP              */
#define IP_PRO_UDP       0x11    /* IP protocol UDP              */

/* TCP header */
#define TCP_HDRSIZE      sizeof(struct tcp_header)
#define TCP_FIN          0x01    /* Finished send data           */
#define TCP_SYN          0x02    /* Synchronize sequence numbers */
#define TCP_RST          0x04    /* Reset the connection         */
#define TCP_PUSH         0x08    /* Push data to the app layer   */
#define TCP_ACK          0x10    /* Acknowledge                  */
#define TCP_URG          0x20    /* Urgent                       */



struct ether_header {
  /* Ethernet header */
  unsigned char      eth_dst_addr[ETH_ADD_LEN]; /* Destination ethernet address	 */
  unsigned char      eth_src_addr[ETH_ADD_LEN]; /* Source ethernet address       */
  unsigned short int eth_type;                  /* EtherType                     */
};

struct ip_header {
  /* IP header */
  unsigned char      ip_hl:4,                   /* Header length                 */
                     ip_v:4;                    /* Version                       */                     
  unsigned char      ip_tos;                    /* Type of service               */
  unsigned short int ip_len;                    /* Total length                  */
  unsigned short int ip_id;                     /* Identification                */  
  unsigned short int ip_flags_off;              /* Flags & offset                */                     
  unsigned char      ip_ttl;                    /* Time to live                  */
  unsigned char      ip_p;                      /* Protocol                      */
  unsigned short int ip_sum;                    /* Checksum                      */
  unsigned char      ip_src[4];                 /* Source address                */
  unsigned char      ip_dst[4];                 /* Destination address           */   	
};

struct tcp_header {
  /* TCP header */
  unsigned short int tcp_src;                   /* Source port                   */
  unsigned short int tcp_dst;                   /* Destination port              */
  unsigned long int  tcp_seq;                   /* Sequence number               */
  unsigned long int  tcp_ack;                   /* Acknowledgement number        */
  unsigned char      tcp_res:4;                 /* Reserved                      */
  unsigned char      tcp_off:4;                 /* Data offset                   */
  unsigned char      tcp_flags;                 /* Flags                         */
  unsigned short int tcp_win;                   /* Windows                       */
  unsigned short int tcp_sum;                   /* Checksum                      */
  unsigned short int tcp_urp;                   /* Urgent pointer                */
};


/* Prototypes */
void usage(char *name);
int check_tcp(LPPACKET lpPacket, unsigned char *etherdst, unsigned char *ethersrc,
			 unsigned char *ipsrc, unsigned char *ipdst,
			 unsigned short int tcp_dst_port, unsigned short int *tcp_src_port,
			 unsigned long int *tcp_ack_num, unsigned short int *tcp_win);

void rst_build(unsigned char *packet, unsigned char *dst_etheraddr, 
			   unsigned char *src_etheraddr, unsigned char *ip_dst_addr, 
			   unsigned char *ip_src_addr, unsigned short int tcp_dst_port, 
			   unsigned short int tcp_src_port, unsigned long int tcp_ack, 
			   unsigned short int tcp_win);

int libnet_in_cksum(u_short *addr, int len);


#endif /* _TCPKILL_H */